/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2016 SSAB EMEA AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 */


package jpwr.app;
import java.io.*;
import java.util.*;


public class FlowLine implements FlowArrayElem {
  FlowPoint p1;
  FlowPoint p2;
  int draw_type;
  int line_width;
  FlowCmn cmn;

  public FlowLine( FlowCmn cmn) {
    this.cmn = cmn;
    p1 = new FlowPoint(cmn);
    p2 = new FlowPoint(cmn);
  }

  @Override
  public int type() {
	 return Flow.eObjectType_Line;
  }

  public void open( BufferedReader reader) {
    String line;
    StringTokenizer token;
    boolean end = false;

    try {
      while( (line = reader.readLine()) != null) {
	token = new StringTokenizer(line);
	int key = new Integer(token.nextToken()).intValue();
	if ( cmn.debug) System.out.println( "line : " + key);

	switch ( key) {
	case Flow.eSave_Line:
	  break;
	case Flow.eSave_Line_draw_type:
	  draw_type  = new Integer(token.nextToken()).intValue();
	  break;
	case Flow.eSave_Line_line_width:
	  line_width  = new Integer(token.nextToken()).intValue();
	  break;
	case Flow.eSave_Line_p1:
	  p1.open( reader);
	  break;
	case Flow.eSave_Line_p2:
	  p2.open( reader);
	  break;
	case Flow.eSave_End:
	  end = true;
	  break;
	default:
	  System.out.println( "Syntax error in FlowLine");
	  break;
	}
	if ( end)
	  break;
      }
    } catch ( Exception e) {
      System.out.println( "IOExeption FlowLine");
    }
  }

  public void draw( FlowPoint p, FlowNodeIfc node, boolean highlight) {
	  int color;
	  
	  switch ( draw_type) {
      case Flow.eDrawType_LineGray:
        color = Plow.COLOR_LIGHTGRAY;
        break;
      case Flow.eDrawType_LineRed:
      case Flow.eDrawType_LineDashedRed:
        color = Plow.COLOR_RED;
        break;
      default:
        color = Plow.COLOR_BLACK;
      }
      if ( highlight)
        color = Plow.COLOR_RED;

      cmn.gdraw.line(true, color, (float)((p1.x + p.x) * cmn.zoom_factor - cmn.offset_x), 
				(float)((p1.y + p.y) * cmn.zoom_factor - cmn.offset_y), 
				(float)((p2.x + p.x) * cmn.zoom_factor - cmn.offset_x), 
				(float)((p2.y + p.y) * cmn.zoom_factor - cmn.offset_y));
	  
	 /*
	  Line2D.Double line = new Line2D.Double( (p1.x + p.x) * cmn.zoom_factor,
						   (p1.y + p.y) * cmn.zoom_factor,
						   (p2.x + p.x) * cmn.zoom_factor,
						   (p2.y + p.y) * cmn.zoom_factor);
      switch ( draw_type) {
      case Flow.eDrawType_LineDashed:
      case Flow.eDrawType_LineDashedRed:
	g.setStroke( new BasicStroke( (float)(cmn.zoom_factor / cmn.base_zoom_factor * line_width),
		     BasicStroke.CAP_BUTT,
		     BasicStroke.JOIN_BEVEL, 0, 
		     new float[] { 4F * line_width, 2F * line_width},
		     0));
	break;
      default:
        g.setStroke( new BasicStroke( (float)(cmn.zoom_factor / cmn.base_zoom_factor * line_width)));
      }
      switch ( draw_type) {
      case Flow.eDrawType_LineGray:
        g.setColor( Color.lightGray);
        break;
      case Flow.eDrawType_LineRed:
      case Flow.eDrawType_LineDashedRed:
        g.setColor( Color.red);
        break;
      default:
        g.setColor( Color.black);
      }
      if ( highlight)
        g.setColor( Color.red);
      g.draw( line);
*/
  }
  @Override
  public boolean getSelect() {
	  return false;
  }
  @Override
  public void setSelect(boolean select) {	  
  }
  public boolean eventHandler(PlowEvent e) {
	  return false;	
  }	

}




