/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2016 SSAB EMEA AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 */


package jpwr.app;
import java.io.*;
import java.util.*;

public class FlowText implements FlowArrayElem {
  FlowPoint p;
  int draw_type;
  int text_size;
  String text;
  FlowCmn cmn;

  public FlowText( FlowCmn cmn) {
    this.cmn = cmn;
    p = new FlowPoint(cmn);
  }

  @Override
  public int type() {
	 return Flow.eObjectType_Text;
  }

  public void open( BufferedReader reader) {
    String line;
    StringTokenizer token;
    boolean end = false;

    try {
      while( (line = reader.readLine()) != null) {
	token = new StringTokenizer(line);
	int key = new Integer(token.nextToken()).intValue();
	if ( cmn.debug) System.out.println( "line : " + key);

	switch ( key) {
	case Flow.eSave_Text:
	  break;
	case Flow.eSave_Text_text_size:
	  text_size = new Integer(token.nextToken()).intValue();
	  break;
	case Flow.eSave_Text_draw_type:
	  draw_type = new Integer(token.nextToken()).intValue();
	  break;
	case Flow.eSave_Text_text:
	  text = token.nextToken();
	  break;
	case Flow.eSave_Text_p:
	  p.open( reader);
	  break;
	case Flow.eSave_End:
	  end = true;
	  break;
	default:
	  System.out.println( "Syntax error in FlowText");
	  break;
	}
	if ( end)
	  break;
      }
    } catch ( Exception e) {
      System.out.println( "IOExeption FlowText");
    }
  }

  public void draw( FlowPoint p0, FlowNodeIfc node, boolean highlight) {
    int tsize;
	int idx = (int) (cmn.zoom_factor/cmn.base_zoom_factor * text_size * 5.8 + 1.0);
    if ( cmn.zoom_factor/cmn.base_zoom_factor * text_size * 6.5 < 2) return;

    int color = Flow.eDrawType_Line;
    if ( highlight)
      color = Flow.eDrawType_LineRed;

    cmn.gdraw.drawText( text, color, idx, 0, (float)((p.x + p0.x) * cmn.zoom_factor - cmn.offset_x), 
			(float)((p.y + p0.y) * cmn.zoom_factor - cmn.offset_y));
  }
  @Override
  public boolean getSelect() {
	  return false;
  }
  @Override
  public void setSelect(boolean select) {	  
  }
  public boolean eventHandler(PlowEvent e) {
	  return false;	
  }	

}








