/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2016 SSAB EMEA AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 */

package jpwr.jop;
import jpwr.rt.*;

/**
   Ge dynamic type Invisible.
*/
public class GeDynInvisible extends GeDynElem {
  String attribute;
  int dimmed;

  boolean attrFound;
  PwrtRefId subid;
  int typeId;
  int p;
  boolean inverted;
  boolean oldValueB;
  int oldValueI;
  String oldValueS;
  boolean firstScan = true;
  boolean cmd = false;
  boolean valueCmd = false;
  int bitNum;
  String command;

  public GeDynInvisible( GeDyn dyn, String attribute, int dimmed) {
    super( dyn, GeDyn.mDynType_Invisible, GeDyn.mActionType_No);
    this.attribute = attribute;
    this.dimmed = dimmed;
  }
  public void connect() {
    String attrName = dyn.getAttrName( attribute);
    if ( attrName.compareTo("") != 0) {
      if ( attrName.toLowerCase().startsWith("$cmd(")) {
	cmd = true;
	valueCmd = false;
	command = attrName.substring(5, attrName.length()-1);
      }
      else {
        GdhrRefObjectInfo ret = dyn.en.gdh.refObjectInfo( attrName);
        if ( ret.evenSts())
	  System.out.println( "DigInvisible: " + attrName);
        else {
	  attrFound = true;
	  p = ret.id;
	  subid = ret.refid;
	  typeId = ret.typeId;
	  inverted = GeDyndata.getAttrInverted( attribute);
	  if ( typeId == GeDyn.eType_Bit) {
	    // Get bit number from name
	    int i1, i2;
	    i1 = attrName.lastIndexOf('[');
	    i2 = attrName.lastIndexOf(']');
	    if ( i1 != -1 && i2 != -1 && i2 > i1) {
	      try {
	        bitNum = Integer.valueOf( attrName.substring( i1+1, i2)).intValue();
	      }
	      catch ( NumberFormatException e) {
		System.out.println( "DigInvisible: " + attrName);
		attrFound = false;
	      }
	    }
	  }
        }
      }
    }
  }
  public void disconnect() {
    if ( attrFound)
      dyn.en.gdh.unrefObjectInfo( subid);
  }
  public void scan() {
    if ( cmd) {
      if ( firstScan) {
	int sts = Jop.executeCommand( dyn.session, command);
	System.out.println( "DynInvisible: " + command + ", value: " + sts);
	if ( sts == 0) {
          if ( dimmed == 0)
            dyn.comp.setVisibility( Ge.VISIBILITY_INVISIBLE);
	  else
            dyn.comp.setVisibility( Ge.VISIBILITY_DIMMED);
	}
	firstScan = false;
      }
      return;
    }
    if ( !attrFound)
      return;
    switch ( typeId) {
    case Pwr.eType_Boolean: {
      boolean value = dyn.en.gdh.getObjectRefInfoBoolean( p);
      if ( !firstScan) {
        if ( value == oldValueB) {
	  // No change since last time
	  return;
        }
      }
      else
        firstScan = false;

      if ( (!inverted && value) || (inverted && !value)) {
        if ( dimmed == 0)
          dyn.comp.setVisibility( Ge.VISIBILITY_INVISIBLE);
        else
          dyn.comp.setVisibility( Ge.VISIBILITY_DIMMED);
      }
      else {
        dyn.comp.setVisibility( Ge.VISIBILITY_VISIBLE);
      }
      dyn.repaintNow = true;
      oldValueB = value;
      break;
    }
    case Pwr.eType_String: {
      String value0;
      value0 = dyn.en.gdh.getObjectRefInfoString( p, typeId);

      if ( firstScan || value0.compareTo( oldValueS) != 0) {
        if ( value0.compareTo("") == 0) {
          if ( dimmed == 0)
            dyn.comp.setVisibility( Ge.VISIBILITY_INVISIBLE);
          else
            dyn.comp.setVisibility( Ge.VISIBILITY_DIMMED);
        }
        else {
          dyn.comp.setVisibility( Ge.VISIBILITY_VISIBLE);
	}
        dyn.repaintNow = true;
	oldValueS = value0;
	if ( firstScan)
	  firstScan = false;
      }
      break;
    }
    case GeDyn.eType_Bit: {
      int value0;
      value0 = dyn.en.gdh.getObjectRefInfoInt( p);

      if ( firstScan || value0 != oldValueI) {
	boolean bitval = (value0 & (1 << bitNum)) != 0;
        if ( ( !inverted && bitval) || ( inverted && !bitval)) {
          if ( dimmed == 0)
            dyn.comp.setVisibility( Ge.VISIBILITY_INVISIBLE);
          else
            dyn.comp.setVisibility( Ge.VISIBILITY_DIMMED);
        }
        else {
          dyn.comp.setVisibility( Ge.VISIBILITY_VISIBLE);
	}
        dyn.repaintNow = true;
	oldValueI = value0;
	if ( firstScan)
	  firstScan = false;
      }
      break;
    }
    }
  }
}








