/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2016 SSAB EMEA AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 */


package jpwr.jop;
import jpwr.rt.*;
import jpwr.jop.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.Timer;
import javax.swing.table.*;
import javax.swing.event.*;

/**
   Ge table component.
*/
public class GeTable extends JScrollPane implements GeComponentIfc, 
		  JopDynamic, ActionListener, ListSelectionListener
{
  Dimension size;
  Object root;
  Timer timer = new Timer(500, this);
  JopSession session;
  public JopEngine en;
  public GeDyn dd = new GeDyn( this);

  JTable table;
  JTable headerColumn;
  int rows;
  int columns;
  int hRow;
  int hColumn;
  JViewport jv;
  int headerRowHeight = 0;

  public boolean focus = false;
  public boolean confirmActive = false;
  public Color  normalColor = null;
  public GeTable component = this;
  public GeTable( JopSession session, int rows, int columns, int hRow, int hColumn)
  {
    this.session = session;
    dd.setSession( session);
    size = new Dimension( 102, 36);
    timer.start();

    this.rows = rows;
    this.columns = columns;
    this.hRow = hRow;
    this.hColumn = hColumn;
      
    table = new JTable( rows, columns - hColumn) {
      public boolean isCellEditable( int row, int col) {
	return false;
      }
    };
    table.setColumnSelectionAllowed( false);
    table.setRowSelectionAllowed( false);
    table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    table.getColumnModel().getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    if ( hRow == 0)
      table.setTableHeader( null);
    else
      table.getTableHeader().setReorderingAllowed(false);

    if ( hColumn == 1) {
      // Set up the header column
      headerColumn = new JTable(rows, 1) {
        public boolean isCellEditable( int row, int col) {
	  return false;
        }
      };
      table.setSelectionModel( headerColumn.getSelectionModel());

      headerColumn.setMaximumSize( new Dimension( 90, 10000));
      headerColumn.setColumnSelectionAllowed( false);
      headerColumn.setRowSelectionAllowed( false);
      // headerColumn.setCellSelectionEnabled( false);

      jv = new JViewport();
      jv.setView( headerColumn);
      jv.setPreferredSize( headerColumn.getMaximumSize());

      table.setAutoResizeMode( JTable.AUTO_RESIZE_OFF);
      headerColumn.setAutoResizeMode( JTable.AUTO_RESIZE_OFF);

      getViewport().setView(table);
      setRowHeader( jv);
    }
    else {
      table.setAutoResizeMode( JTable.AUTO_RESIZE_OFF);
      getViewport().setView(table);
    }
  }

  public void setHeaderText( int idx, String text) {
    if ( idx == 0 && hColumn == 1) {
      // Header of header row is not supported...
    }
    else if ( hRow != 0)
      table.getColumnModel().getColumn(idx - hColumn).setHeaderValue( text);
  }
  public void setColumnWidth( int idx, int width) {
    if ( idx == 0 && hColumn == 1) {
      headerColumn.getColumnModel().getColumn(0).setPreferredWidth( width);
      jv.setPreferredSize( new Dimension( width, 10000));
    }
    else
      table.getColumnModel().getColumn(idx - hColumn).setPreferredWidth( width);
  }
  public void setRowHeight( int height) {
    table.setRowHeight( height);
    if ( hColumn == 1)
      headerColumn.setRowHeight( height);
  }
  public void setHeaderRowHeight( int headerRowHeight) {
    if ( hRow == 1) {
      Dimension d = table.getTableHeader().getPreferredSize();
      d.height = headerRowHeight;
      table.getTableHeader().setPreferredSize( d);
      if ( hColumn == 1) {
      	Dimension hd = headerColumn.getTableHeader().getPreferredSize();
     	hd.height = headerRowHeight;
	headerColumn.getTableHeader().setPreferredSize( hd);
      }
    }
  }
  public void setFont( Font font) {
    if ( table != null)
      table.setFont( font);
    if ( hColumn == 1 && headerColumn != null) {
      headerColumn.setFont( font);
      if ( hRow == 1)
	headerColumn.getTableHeader().setFont( font);
    }
    if ( hRow == 1)
      table.getTableHeader().setFont( font);
  }

  public void setValueAt( String value, int row, int column) {
    if ( column == 0 && hColumn == 1)
      headerColumn.setValueAt( value, row, column);
    else
      table.setValueAt( value, row, column - hColumn);
  }

  public int getSelectedRow() {
    int row = table.getSelectedRow();
    if ( row == -1 && hColumn == 1)
      row = headerColumn.getSelectedRow();
    return row;
  }
  public int getSelectedColumn() {
    int column;

    if ( hColumn == 1)
      // Always return header column
      column = 0;
    else
      column = table.getSelectedColumn();
    return column;
  }

  public void actionPerformed(ActionEvent e) {
    boolean engine_found = false;

    root = getTopLevelAncestor();
    if ( root != null) {
      if ( root instanceof JopApplet)
	session = ((JopApplet)root).session;
      else if ( root instanceof JopFrame)
	session = ((JopFrame) root).session;
      en = session.getEngine();

      if ( !en.isReady())
        return;
      en.add(this);
      engine_found = true;
    }

    if ( engine_found) {
      timer.stop();
      timer = null;

      if ( en.gdh.isAuthorized( dd.access)) {
        table.getSelectionModel().addListSelectionListener( this);
        table.getColumnModel().getSelectionModel().addListSelectionListener( this);

        table.addMouseListener(new MouseAdapter() {
          public void mouseReleased(MouseEvent e) {
	    if ( e.isPopupTrigger())
	      dd.action( GeDyn.eEvent_MB3Press, e);
	  }

          public void mousePressed(MouseEvent e) {
	    if ( e.isPopupTrigger())
	      dd.action( GeDyn.eEvent_MB3Press, e);
	  }
        });
	if ( hColumn == 1) {
          headerColumn.addMouseListener(new MouseAdapter() {
            public void mouseReleased(MouseEvent e) {
	      if ( e.isPopupTrigger())
	        dd.action( GeDyn.eEvent_MB3Press, e);
	    }

            public void mousePressed(MouseEvent e) {
	      if ( e.isPopupTrigger())
	        dd.action( GeDyn.eEvent_MB3Press, e);
	    }
          });
	}
      }
    }
  }

  public void valueChanged(ListSelectionEvent e) {
    MouseEvent me = new MouseEvent( this, 0,0,0,0,0,0,false);
    dd.action( GeDyn.eEvent_MB1Click, me);
  }

  // GeComponents Ifc
  public void tsetFillColor( int fillColor) { 

    this.fillColor = fillColor;
    normalColor = GeColor.getColor( fillColor, colorTone,
	 colorShift, colorIntensity, colorBrightness, colorInverse, fillColor, false);
    table.setBackground( normalColor);
    if ( hColumn == 1)
      headerColumn.setBackground( normalColor);
  }
  public void tsetColorTone( int colorTone) {
    this.colorTone = colorTone;
    normalColor = GeColor.getColor( fillColor, colorTone,
	 colorShift, colorIntensity, colorBrightness, colorInverse, GeColor.NO_COLOR, false);
    table.setBackground( normalColor);
    if ( hColumn == 1)
      headerColumn.setBackground( normalColor);
  }
  public void tsetBorderColor( int borderColor) {}
  public void tsetTextColor( int borderColor) {}
  public void setColorInverse( int colorInverse) {} 
  public void resetFillColor() { 
    fillColor = originalFillColor;
    normalColor = GeColor.getColor( fillColor, colorTone,
	 colorShift, colorIntensity, colorBrightness, colorInverse, fillColor, false);
    table.setBackground( normalColor);
    if ( hColumn == 1)
      headerColumn.setBackground( normalColor);
  }
  public void resetColorTone() { 
    colorTone = originalColorTone;
    normalColor = GeColor.getColor( fillColor, colorTone,
	 colorShift, colorIntensity, colorBrightness, colorInverse, fillColor, false);
    table.setBackground( normalColor);
    if ( hColumn == 1)
      headerColumn.setBackground( normalColor);
  }
  public void resetBorderColor() {}
  public void resetTextColor() {}
  public void setVisibility( int visibility) {}
  public String getAnnot1() { return "";}
  public void setAnnot1( String s) {}
  public void setLastPage() {}
  public void setFirstPage() {}
  public void setPage( int page) {}
  public int setNextPage() { return 1;}
  public int setPreviousPage() { return 1;}
  public Object getDd() { return dd;}
  public void setFillLevel( float fillLevel) {}
  public void setLevelDirection( int levelDirection) {}
  public void setLevelColorTone( int levelColorTone) {}
  public void setLevelFillColor( int levelFillColor) {}

  Font cellFont = new Font("Helvetica", Font.BOLD, 14);
  public void setCellFont( Font font) { cellFont = font; setFont(font);}
  public Font getCellFont() { return cellFont;}
  public int fillColor = 9999;
  public int originalFillColor = 9999;
  public int borderColor = 9999;
  public int originalBorderColor = 9999;
  public int colorTone = 0;
  public int originalColorTone = 0;
  public int colorShift = 0;
  public int originalColorShift = 0;
  public int colorBrightness = 0;
  public int originalColorBrightness = 0;
  public int colorIntensity = 0;
  public int originalColorIntensity = 0;
  public int colorInverse = 0;
  public int originalColorInverse = 0;
  public void setColorTone( int colorTone) {
    this.colorTone = colorTone;
    originalColorTone = colorTone;
  }
  public int getColorTone() {
    return colorTone;
  }
  public void setColorShift( int colorShift) {
    this.colorShift = colorShift;
    originalColorShift = colorShift;
  }
  public int getColorShift() {
    return colorShift;
  }
  public void setColorBrightness( int colorBrightness) {
    this.colorBrightness = colorBrightness;
    originalColorBrightness = colorBrightness;
  }
  public int getColorBrightness() {
    return colorBrightness;
  }
  public void setColorIntensity( int colorIntensity) {
    this.colorIntensity = colorIntensity;
    originalColorIntensity = colorIntensity;
  }
  public int getColorIntensity() {
    return colorIntensity;
  } 
  public void setFillColor( int fillColor) {
    this.fillColor = fillColor;
    this.originalFillColor = fillColor;
  }
  public int getFillColor() {
    return fillColor;
  }
  public void setBorderColor( int borderColor) {
    this.borderColor = borderColor;
    this.originalBorderColor = borderColor;
  }
  public int getBorderColor() {
    return borderColor;
  }
  public double rotate;
  public void setRotate( double rotate) {}
  public double getRotate() { return rotate;}

  public Object dynamicGetRoot() {
    return root;
  }
  public void dynamicOpen() {
    if ( en.isInstance())
      dd.setInstance( en.getInstance());

    dd.connect();

    normalColor = GeColor.getColor( fillColor, colorTone,
	 colorShift, colorIntensity, colorBrightness, colorInverse, fillColor, false);
    table.setBackground( normalColor);
    if ( hColumn == 1)
      headerColumn.setBackground( normalColor);
  }
  public void dynamicClose() {
    dd.disconnect();
  }
  public void dynamicUpdate( boolean animationOnly) {
    if ( animationOnly || focus)
      return;

    dd.scan();

  }

  public void repaintForeground() {
    Graphics g = getGraphics();
    if ( g == null) {
      System.out.println("repaintForeground: can't get Graphic object");
      return;
    }
    paintComponent(g);
    paintChildren(g);
  }

}















